<!--
Project: Content Management System.
Component: Reader/Parser (File 2 of 2).
Filename: "parse_xml.php".
Purpose: Parse the XML document and display the assets according to type.
-->

<?php
/*
The $rootPath variable is used to indicate the full file path to the asset files on your hard drive.  This is necessary for the parser to check that the asset file exists.  This check occurs later in the script. The $metaDataDirectory variable holds the value for the directory containing the XML files.
*/
$rootPath = "c:\\xampp\\htdocs\\xmlparsers\\cms\\";
$metaDataDirectory = "metadata\\";
?>

<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
 <head>
  <title> Simple Content Management System </title>
 </head>

 <body>

<h1>Simple Content Management System</h1>
<h2>Asset Collection</h2>

<?php
/*
This section of code will check to see if an input file has been specified through the URI (e.g. parse_xml.php?inputFilename=test.xml).  If not, the parser will display an error message and will exit.  "Exit" is a special PHP function that will exit the script completely and display a message if a failure is encountered.
*/
if (isset($_GET["inputFilename"]))
	$inputFilename = $_GET["inputFilename"];
else
	exit("No input file is specified.  Please specify an input file using by appending a question mark at the end of this filename and then including the name of the XML file (e.g. parse_xml.php?inputFilename=test.xml).  Or, you can also use the built-in directory browser <a href=\"select_xml.php\">available here</a>.");
?>

<!-- This will display the name of the current document being parsed -->
<hr />
<p>Parsing document named "<?php echo $inputFilename; ?>".</p>

<?php

/*
The startElement function is the function that will be called whenever a beginning XML tag is encountered.  For instance, if the parser is currently parsing an element named "<asset_collection>", the parser will jump to the block of code specified by the case "asset_collection" condition.  In PHP, the switch statement works like a series of IF statements where the same value is being compared in each condition.  In this case, we are looking at the $name variable's value and printing code to the browser depending on which XML element is currently being processed.  
*/
function startElement($parser, $name, $attributes)
	{
	switch($name)
		{
		case "xml":
			return;
			break;
		case "asset_collection":
			echo "<table border=\"1\" width=\"640\">";
			echo "<tr>";
			echo "<th>Id</th>";
			echo "<th>Filename</th>";
			echo "<th>Asset name</th>";
			echo "<th>Categories</th>";
			echo "<th>Years Used</th>";
			echo "<th>Media</th>";
			echo "<th>Description</th>";
			echo "</tr>";
			break;
		case "asset":
			echo "<tr>";
			break;
		default:
			echo "<td>";
			break;
		}
	}

/*
This function is used to display the character data that is encapsulated within each element tag.  Global is a special keyword in PHP that enables a function to access a variable defined outside the scope of the function.  In this case, we need to use the $rootPath variable that is defined at the top of this script.  Next, we use the eregi function to scan the $value variable and look for any files that end with .jpg, .gif, .doc, .pdf, or .txt.  These will be the asset files themselves rather than the metadata annotations.  We are checking these files because we need to do some special error checking to make sure the assets actually exist before we provide links to them (see next comment below).
*/
function characterData($parser, $value)
	{
	global $rootPath;
	if (eregi("[.]jpg|[.]gif|[.]doc|[.]pdf|[.]txt",$value))
		{
		echo $value;
		$longFileName = $rootPath."assets\\".$value;
		
		/*
		This segment of code will check to verify that a file actually exists on the hard disk drive before providing a link to it.  If a file is missing, the text "missing" will be displayed instead.  This is an advantage to using a custom designed parser rather than simple XSL transformations in that we can use built-in functions like this for additional error checking.
		*/  

		if (file_exists($longFileName))
			echo "<br /><a href=\"assets/".$value."\" target=\"_blank\">$value</a>";
		else
			echo "<br />(missing)";
		}
	else
		{
		echo $value;
		}
	}

/*
The endElement function is the function that will be called whenever an ending XML tag is encountered.  For instance, if the parser is currently parsing an element named "</asset_collection>", the parser will jump to the block of code specified by the case "asset_collection" condition.  In PHP, the switch statement works like a series of IF statements where the same value is being compared in each condition.  In this case, we are looking at the $name variable's value and printing code to the browser depending on which XML element is currently being processed.  If </asset_collection> is parsed, this tells us that the collection is now entirely scanned and we can go ahead and close the table using the HTML tag </table>.  
*/
function endElement($parser, $name)
	{
	switch($name)
		{
		case "xml":
			return;
			break;
		case "asset_collection":
			echo "</table>";
			break;
		case "asset":
			echo "</tr>";
			break;
		default:
			echo "</td>";
			break;
		}
	}


/*
The function xml_parser_create() is used to create a new parser in PHP.  It then returns a "resource handle" which is a special variable that can be referenced from that point on when working with the parser.  In this case, our resource handle is stored in a variable named $xml.
*/
$xml = xml_parser_create();

/*
The XML_OPTION_CASE_FOLDING option is used to make sure that the parser does not transform all elements into uppercase when parsing an XML document.  Since XML is case-sensitive, leaving case folding on, which is the default, could cause problems when parsing the file.  Here, setting this option to "false" means that the parser will leave lowercase or mixed case elements as they were originally created. 
*/
xml_parser_set_option($xml, XML_OPTION_CASE_FOLDING, false);

/*
This function is used to specify functions that should be called when starting elements and ending elements are encountered in the XML hierarchy.  We have defined these functions, named startElement() and endElement() in the top portion of this code.  Anytime a new beginning element (e.g. <asset>) is encountered, the startElement function will be called and the code can perform conditional actions based on the name of the element.  Then, when an ending element (e.g. </asset>) is processed, the endElement function is called.
*/
xml_set_element_handler($xml, "startElement", "endElement");

/*
This function is very similar to the one above, but instead of creating a function to handle elements, it creates a function to handle the character data itself.  For example, when values that exist between XML element tags are processed, these values are sent to the function defined here.  In this case, we named it characterData().
*/
xml_set_character_data_handler($xml, "characterData");

/*
This block of code will read the file into memory and then attempt to parse it.  If the file is not found, an error message will be displayed to the user.  Similarly, if the file is unable to be parsed (which could be due to malformed XML content or file corruption) a secondary error message will be displayed.  The ampersand (@) in front of the file_get_contents() function is used to suppress warning messages.  Since we are already providing error messages in the case of a failed operation, we don't need to clutter the user's browser with additional warnings from PHP.
*/  
if ($xmlString = @file_get_contents($metaDataDirectory.$inputFilename) or die("Input file not available."))
	{
	xml_parse($xml, $xmlString);
	}
else
	{
	echo "Error.  Unable to load input file for parsing.<br />";
	}

?>

<br />

<a href="select_xml.php">Select Different Asset Database</a>&nbsp;&nbsp;
<a href="upload_asset.html">Begin New Asset Collection</a>&nbsp;&nbsp;

</body>

</html>

<!-- End of "parse_xml.php" file -->