<!--
Project: Content Management System.
Component: Writer (File 2 of 5).
Filename: "upload_asset.php".
Purpose: Process the uploaded asset, rename it, and move it to the assets directory.
-->

<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
 <head>
  <title> Simple Content Management System </title>
 </head>

 <body>

<h1>Simple Content Management System</h1>
<h2>Step 1b: Upload Results Page</h2>

<?php

/* 
The $bufferFilename variable is used to store the name of the temporary file we will be writing to. For this exercise, this file needs to be in the same directory as this script, but it could be changed by including the directory path as part of this variable.
*/

$bufferFilename = "asset_buffer.txt";

/*
This array defines the different types of files our CMS supports.
*/

$supportedTypes = array();
$supportedTypes[] = "image/gif";
$supportedTypes[] = "image/jpeg";
$supportedTypes[] = "image/pjpeg";
$supportedTypes[] = "application/msword";
$supportedTypes[] = "text/plain";
$supportedTypes[] = "application/pdf";

/*
This array is used to map the appropriate file extensions onto our assets.
*/

$fileSuffixes = array();
$fileSuffixes["image/gif"] = ".gif";
$fileSuffixes["image/pjpeg"] = ".jpg";
$fileSuffixes["image/jpeg"] = ".jpg";
$fileSuffixes["application/msword"] = ".doc";
$fileSuffixes["text/plain"] = ".txt";
$fileSuffixes["application/pdf"] = ".pdf";

/*
This function will open the asset "buffer" text file to obtain the index id number of the last added asset.
*/
function getLastAssetNumber($filename)
	{
	/* open in "read only" mode.  If file doesn't exist, the CMS will treat this asset as the first item in the collection. */
	if ($fp = @fopen($filename, "r"))
		{  
		/* read the last asset number */
		$lastAssetNumber = fgets($fp); 
		}

	/* if this number is not available, then return zero as the last asset number */
	if (empty($lastAssetNumber)) $lastAssetNumber = 0;

	/* this will be either zero (new collection) or some number N of previously existing assets */
	return($lastAssetNumber);
	}

/*
This function will open the asset "buffer" text file and write the index id number of the most recently added asset.
*/
function writeNewAssetNumber($filename, $newNum)
	{
	$fp = fopen($filename, "w"); // open in "overwrite" mode
	$bytes_written = fputs($fp, $newNum."\n");
	}

/*
The first thing we will do when uploading our asset file is to access the temporary filename from the special $_FILES array that is automatically created by PHP
based on the "FILE" form element submitted on the prior page.  This filename is stored in an index named [x]["tmp_name"] where x is the name of the form field specified in the form.  In this case, it is named "asset".  We can also obtain the type of file using the ["type"] index (this value may be slightly different in browsers other than Internet Explorer).
*/

$originalFileName = $_FILES["asset"]["name"];
$fileName = $_FILES["asset"]["tmp_name"];
$fileType = $_FILES["asset"]["type"];

/*
Here, we will get the file suffix (.doc, .pdf, etc.) which can help to minimize problems with certain browsers not recognizing the default file types correctly. First, the explode function will turn the file name into an array split by where the period appears.  For example, if the filename was "information.doc", then $temp[0] would contain the value "information" and $temp[1] would contain the value "doc".  So, we store the suffix into a variable named $fileSuffix.
*/

$temp = explode(".",$originalFileName);
$fileSuffix = $temp[1];

/*
Next, we need to make sure this variable contains data.  If not, we provide a link back to the previous page and ask the user to enter data for the filename field.  The $errors variable will keep track of total errors and if this variable is greater than zero, the user will be asked to go back and provide any missing data.
*/

if (empty($fileName))
	{
	echo "A valid filename is required!<br />";
	$errors++;
	}

/* 
Here is the part where we check the total errors and redirect the user if need be.
*/

if ($errors == 1)
	{
	echo "There was an error with your data.<br />";
	echo "Please <a href=\"upload_asset.html\">go back</a> to correct it!<br />";
	}
else
	{ 

	/* 
	This code will check to make sure the file type is supported for our content management system (jpg, gif, doc, txt, or pdf) 
	*/
	
	/* If so, we need to try and process the uploaded file. */
	if (in_array($fileType, $supportedTypes) || ($fileSuffix == "doc"))
		{
		/* find the number of the last added asset */
		$lastAssetNumber = getLastAssetNumber($bufferFilename);

		/* add one to it so that it will be unique */ 
		$currentAssetNumber = $lastAssetNumber + 1;  

		/* tell the user the calculated file type */
		echo "File is type: $fileType.<br />"; 

		/* find the correct file suffix to use (e.g., .pdf or .doc) */		
		$newFileSuffix = $fileSuffixes[$fileType];

		/* if unable to find new file suffix, use the existing file suffix */
		if (empty($newFileSuffix)) $newFileSuffix = ".".$fileSuffix; 

		/* create a new filename based on assetX.Y (X=new id number, Y=suffix) */
		$newFileName = "asset".$currentAssetNumber.$newFileSuffix;  

		/*
		Assuming everything is okay, move the uploaded file to the assets directory and provide a link to step 2. 
		*/
		if (move_uploaded_file($fileName, "assets\\$newFileName"))
			{
			echo "Asset added as ".$newFileName.".";

			/* Call the function we defined at the top of this page to write the asset number to a text file */
			writeNewAssetNumber($bufferFilename, $currentAssetNumber);
			?>

			<!-- This code will display a link to the next step in the process, which is adding metadata. -->
			<p>Click here to go to Step 2 and <a href="add_asset_metadata.php?asset_id=<?php echo $currentAssetNumber; ?>&asset_filename=<?php echo $newFileName; ?>">add metadata for this asset</a>.</p>
			<?php
			}
		else
			{
			/* If the file is unable to be uploaded, provide an error message */
			echo "Unable to add asset ".$fileName.".";
			} 

		}
	else
		{
		/* If there is a problem with the file type, display an error message for the user. */
		echo "Sorry, but $fileType is not a supported file type.<br />";
		}
	}

?>

</body>

</html>

<!-- End of "upload_asset.php" file -->